@@ -5,22 +5,24 @@
  *  Copyright  1993 metrowerks inc.  All rights reserved.
  * Modified for Kerberos5 Mac port to include compile options
  */
+ 
+#include "autoconf.h"
 
 /*
  * Add the compile flag switches for kerberos compile
  */
 #define KRB5 1
 
-#define SIZEOF_INT 4
-#define SIZEOF_SHORT 2
 #define HAVE_SRAND
 #define NO_PASSWORD
 #define HAVE_LABS
 #define ANSI_STDIO
+#define USE_CCAPI
 
 #include <unix.h>
 #include <ctype.h>
 #include <SocketErrors.h>
+#include <unistd.h>
 
 #define PROVIDE_RSA_MD4
 #define PROVIDE_RSA_MD5
@@ -35,12 +37,11 @@
 
 #define NO_SYS_TYPES_H
 #define NO_SYS_STAT_H
-#define HAVE_STDLIB_H 1
 
 //jfm need to reimplement
 #define mktemp(a)
 
 enum {
 EROFS  = 30,
-ENFILE = 23
+/*ENFILE = 23*/
 };
